@@ -50,16 +50,37 @@ pub fn outter_function(arr: &WTokens) -> WFuncPair {
     results
 }
 
-pub fn bracket_pairs(arr: &WTokens, initial_pos: &usize) -> Option<usize> {
+pub fn bundle_groups(mut arr: WTokens) -> WTokens {
+    let first = first_special_instance("{".to_string(), &arr);
+    let second = match first {
+        Some(initial_pos) => special_pairs(("{".to_string(), "}".to_string()), &arr, &initial_pos),
+        None => None
+    };
+
+    match (first, second) {
+        (Some(x), Some(y)) => {
+            let token_group = Token::Group(arr[x+1..y].to_vec());
+            arr.splice(x..y+1, vec![token_group]);
+            match first_special_instance("{".to_string(), &arr) {
+                Some(_) => bundle_groups(arr),
+                None => arr
+            }
+        }
+        (None, None) => arr,
+        _ => panic!("Invalid grouping!")
+    }
+}
+
+pub fn special_pairs(tokens: (String, String), arr: &WTokens, initial_pos: &usize) -> Option<usize> {
     let mut counter = 0;
     let mut next_open = 0;
 
     for (i, token) in arr[initial_pos.clone() + 1..].iter().enumerate() {
         match token {
             Token::Special(value) => {
-                if value == ")" {
+                if value == &tokens.1 {
                     return Some(initial_pos + i + 1);
-                } else if value == "(" {
+                } else if value == &tokens.0 {
                     next_open = initial_pos + i;
                     counter += 1;
                     break;
@@ -72,9 +93,9 @@ pub fn bracket_pairs(arr: &WTokens, initial_pos: &usize) -> Option<usize> {
     for (i, token) in arr[next_open..].iter().enumerate() {
         match token {
             Token::Special(value) => {
-                if value == "(" {
+                if value == &tokens.0 {
                     counter += 1;
-                } else if value == ")" {
+                } else if value == &tokens.1 {
                     counter -= 1;
                 }
 
@@ -89,11 +110,11 @@ pub fn bracket_pairs(arr: &WTokens, initial_pos: &usize) -> Option<usize> {
     None
 }
 
-pub fn get_first_bracket_open(arr: &WTokens) -> Option<usize> {
+pub fn first_special_instance(special: String, arr: &WTokens) -> Option<usize> {
     for (i, token) in arr.iter().enumerate() {
         match token {
             Token::Special(value) => {
-                if value == "(" {
+                if value == &special {
                     return Some(i);
                 } else {
                     continue;
@@ -107,12 +128,27 @@ pub fn get_first_bracket_open(arr: &WTokens) -> Option<usize> {
 }
 
 pub fn wfunc(function: &WTokens, arr: &WTokens, state: &HashMap<String, WTokens>) -> WTokens {
-    let has_remaining_param = function.iter().any(|x| match x {
+    fn release_groups(arr: &WTokens) -> WTokens {
+        let mut released = arr.clone();
+
+        for (i, token) in arr.iter().enumerate() {
+            if let Token::Group(group) = token {
+                released.splice(i..i+1, group.clone());
+                release_groups(&released);
+            }
+        }
+
+        released
+    }
+
+    let released_function = release_groups(function);
+
+    let has_remaining_param = released_function.iter().any(|x| match x {
         Token::Parameter(FunctionParameter::Remaining) => true,
         _ => false,
     });
 
-    let max_param: usize = function.iter().fold(0, |acc, x| match x {
+    let max_param: usize = released_function.iter().fold(0, |acc, x| match x {
         Token::Parameter(FunctionParameter::Exact(x)) => {
             if acc < x + 1 {
                 x + 1
@@ -123,21 +159,28 @@ pub fn wfunc(function: &WTokens, arr: &WTokens, state: &HashMap<String, WTokens>
         _ => acc,
     });
 
-    let mut buffer: WTokens = Vec::new();
-
-    for token in function {
-        match token {
-            Token::Parameter(FunctionParameter::Exact(x)) => buffer.push(match arr.get(*x) {
-                Some(y) => y.clone(),
-                None => continue,
-            }),
-            Token::Parameter(FunctionParameter::Remaining) => {
-                buffer.append(&mut arr[max_param..].to_vec())
+    fn map_parameters(mut buffer: WTokens, function: &WTokens, arr: &WTokens, max_param: usize) -> WTokens {
+        for token in function {
+            match token {
+                Token::Parameter(FunctionParameter::Exact(x)) => buffer.push(match arr.get(*x) {
+                    Some(y) => y.clone(),
+                    None => continue,
+                }),
+                Token::Parameter(FunctionParameter::Remaining) => {
+                    buffer.append(&mut arr[max_param..].to_vec())
+                },
+                Token::Group(x) => {
+                    buffer.push(Token::Group(map_parameters(vec![], x, arr, max_param)))
+                },
+                _ => buffer.push(token.clone()),
             }
-            _ => buffer.push(token.clone()),
         }
+
+        buffer
     }
 
+    let buffer = map_parameters(vec![], function, arr, max_param);
+
     let mut result = eval(buffer, &state);
 
     if has_remaining_param {
